local MakePlayerCharacter = require "prefabs/player_common"

local assets =
{
    Asset("SCRIPT", "scripts/prefabs/player_common.lua"),
    Asset("SOUND", "sound/wathgrithr.fsb"),
}

local prefabs =
{
    "spear_wathgrithr",
    "wathgrithrhat",
    "wathgrithr_spirit",
}

local start_inv =
{
    "spear_wathgrithr",
    "wathgrithrhat",
    "meat",
    "meat",
    "meat",
    "meat",
}

local smallScale = 0.5
local medScale = 0.7
local largeScale = 1.1

local function spawnspirit(inst, x, y, z, scale)
    local fx = SpawnPrefab("wathgrithr_spirit")
    fx.Transform:SetPosition(x, y, z)
    fx.Transform:SetScale(scale, scale, scale)
end

local function IsValidVictim(victim)
    return victim ~= nil
        and not ((victim:HasTag("prey") and not victim:HasTag("hostile")) or
                victim:HasTag("veggie") or
                victim:HasTag("structure") or
                victim:HasTag("wall") or
                victim:HasTag("companion"))
        and victim.components.health ~= nil
        and victim.components.combat ~= nil
end

local function onkilled(inst, data)
    local victim = data.victim
    if IsValidVictim(victim) then
        -- local delta = victim.components.combat.defaultdamage * 0.25
        -- inst.components.health:DoDelta(delta, false, "battleborn")
        -- inst.components.sanity:DoDelta(delta)

        if not victim.components.health.nofadeout and (victim:HasTag("epic") or math.random() < .1) then
            local time = victim.components.health.destroytime or 2
            local x, y, z = victim.Transform:GetWorldPosition()
            local scale = (victim:HasTag("smallcreature") and smallScale)
                        or (victim:HasTag("largecreature") and largeScale)
                        or medScale
            inst:DoTaskInTime(time, spawnspirit, x, y, z, scale)
        end
    end
end

local BATTLEBORN_STORE_TIME = 3
local BATTLEBORN_DECAY_TIME = 5
local BATTLEBORN_TRIGGER_THRESHOLD = 1

local function onattack(inst, data)
    local victim = data.target
    if not inst.components.health:IsDead() and IsValidVictim(victim) then
        local damage = data.weapon ~= nil and data.weapon.components.weapon.damage or inst.components.combat.defaultdamage
        local damageAfter = damage <= 0 and 0 or damage * inst.DamageMulti
		local healthFactor = 1 - inst.components.health:GetPercent()
		local sanityFactor = 1 - inst.components.sanity:GetPercent()
        --math and clamp does account for 0 and infinite cases
        local delta = math.clamp(damageAfter * (0.05 * (healthFactor + sanityFactor)), .33, 530)

        --decay stored battleborn
        if inst.battleborn > 0 then
            local dt = GetTime() - inst.battleborn_time - BATTLEBORN_STORE_TIME
            if dt >= BATTLEBORN_DECAY_TIME then
                inst.battleborn = 0
            elseif dt > 0 then
                local k = dt / BATTLEBORN_DECAY_TIME
                inst.battleborn = Lerp(inst.battleborn, 0, k * k)
            end
        end

        --store new battleborn
        inst.battleborn = inst.battleborn + delta
        inst.battleborn_time = GetTime()

        --consume battleborn if enough has been stored
        if inst.battleborn > BATTLEBORN_TRIGGER_THRESHOLD then
            inst.components.health:DoDelta(inst.battleborn, true, "battleborn")
            inst.components.sanity:DoDelta(inst.battleborn)
            inst.battleborn = 0
        end
    end
end

local function ondeath(inst)
    inst.battleborn = 0
end

local function OnHealthDelta(inst, data)
	--when enraged (< 20 health), natural defense goes down to normal, bonus damage up to 10x
	local tenPercent = inst.components.health.maxhealth / 10
	local defenseMulti = 1
	if inst.components.health.currenthealth < tenPercent then
		inst.BonusMulti = (1 - (inst.components.health.currenthealth / tenPercent)) * 4
		--at 5%, natural defense is 0 like other characters
		defenseMulti = (inst.components.health.currenthealth / (tenPercent / 2)) - 1
	else
		inst.BonusMulti = 0
	end
	--
    inst.healthDamageMulti = .25 * (1 - data.newpercent)
	inst.DamageMulti = (1 + inst.healthDamageMulti + inst.sanityDamageMulti + inst.BonusMulti) * inst.hungerDamageMulti
	inst.components.combat.damagemultiplier = inst.DamageMulti
	--
	--natural defense
	inst.components.health:SetAbsorptionAmount(.25 * defenseMulti)
end

local function OnSanityDelta(inst, data)
	--
    inst.sanityDamageMulti = .25 * (1 - data.newpercent)
	inst.DamageMulti = (1 + inst.healthDamageMulti + inst.sanityDamageMulti + inst.BonusMulti) * inst.hungerDamageMulti
	inst.components.combat.damagemultiplier = inst.DamageMulti
	--
end

local function OnHungerDelta(inst, data)
	--decreases damage output to 0 if hunger is 0, decrease starts at 40
	inst.hungerDamageMulti = math.clamp(inst.components.hunger.current / 40, 0, 1);
	inst.DamageMulti = (1 + inst.healthDamageMulti + inst.sanityDamageMulti + inst.BonusMulti) * inst.hungerDamageMulti
	inst.components.combat.damagemultiplier = inst.DamageMulti
end

local function OnAttacked(inst, data)
	--damage sanity when attacked
	--inst.components.sanity:DoDelta(-1 * data.damage)
	--damage return 
	if data.attacker and data.attacker.components and data.attacker.components.combat then
		local returnMulti = math.clamp(inst.DamageMulti - 1, 0, 1)
		data.attacker.components.combat:GetAttacked(inst, data.damage * returnMulti)
	end
end

local function common_init(inst)
    inst:AddTag("valkyrie")

    inst.components.talker.mod_str_fn = Umlautify
end

local function master_init(inst)
    inst.talker_path_override = "dontstarve_DLC001/characters/"

    inst.components.eater:SetDiet({ FOODGROUP.OMNI }, { FOODTYPE.MEAT, FOODTYPE.GOODIES })
	inst.components.eater.strongstomach = true

    inst.components.health:SetMaxHealth(TUNING.WATHGRITHR_HEALTH)
    inst.components.hunger:SetMax(TUNING.WATHGRITHR_HUNGER)
    inst.components.sanity:SetMax(TUNING.WATHGRITHR_SANITY)
    inst.components.combat.damagemultiplier = TUNING.WATHGRITHR_DAMAGE_MULT
    inst.components.health:SetAbsorptionAmount(TUNING.WATHGRITHR_ABSORPTION)

    inst:ListenForEvent("killed", onkilled)
    inst:ListenForEvent("onattackother", onattack)
    inst:ListenForEvent("death", ondeath)
	inst:ListenForEvent("healthdelta", OnHealthDelta)
	inst:ListenForEvent("sanitydelta", OnSanityDelta)
	inst:ListenForEvent("hungerdelta", OnHungerDelta)
	inst:ListenForEvent("attacked", OnAttacked)

    inst.battleborn = 0
    inst.battleborn_time = 0
	inst.healthDamageMulti = 1
	inst.sanityDamageMulti = 1
	inst.hungerDamageMulti = 1
	inst.DamageMulti = 1
	inst.BonusMulti = 0
end

return MakePlayerCharacter("wathgrithr", prefabs, assets, common_init, master_init, start_inv)
